﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Runtime.Serialization;
using System.Windows.Forms;
using System.Xml.Serialization;

namespace Masteroppgave.Model
{
    
    public abstract class NodeModel : TreeNode, ISerializable
    {
        
        public string _description, _type, _childtype;
        
        public List<NodeModel> _children;

        /// <summary>
        /// Main constructor
        /// </summary>
        /// <param name="name">Name of the node</param>
        /// <param name="description">Description of the node</param>
        public NodeModel(string name, string description)
            : base(name)
        {
            // For new empty items
            if (name.Equals("")) Name = "New item";
            if (description.Equals("")) _description = "Enter description here";
            else _description = description;
            
            _children = new List<NodeModel>();
            
            ToolTipText = _description;
            
        }

        /// <summary>
        /// Empty constructor
        /// </summary>
        public NodeModel()
            : this("New item", "Enter description here")
        {
            
        }

        /// <summary>
        /// Used for serialization
        /// </summary>
        /// <param name="info"></param>
        /// <param name="ctxt"></param>
        public NodeModel(SerializationInfo info, StreamingContext ctxt)
        {
            //Get the values from info and assign them to the appropriate properties

            Name = (String)info.GetValue("Name", typeof(string));
            Description = (String)info.GetValue("Descr", typeof(string));
            _children = (List<NodeModel>)info.GetValue("children", typeof(List<NodeModel>));
            ToolTipText = Description;
            
        }

        public void GetObjectData(SerializationInfo info, StreamingContext ctxt)
        {
            info.AddValue("Name", Name);
            info.AddValue("Descr", Description);
            info.AddValue("children", _children);
        }

        /// <summary>
        /// Adds a child to the node
        /// </summary>
        /// <param name="name">Name of the child</param>
        /// <param name="description">Description of the child</param>
        public abstract NodeModel AddChild(string name, string description);

        /// <summary>
        /// Adds an empty child to the node
        /// </summary>
        /// <returns></returns>
        public abstract NodeModel AddEmptyChild();

        /// <summary>
        /// Removes/deletes a child from the node
        /// </summary>
        /// <param name="node">Name of the child</param>
        public void RemoveChild(NodeModel node)
        {
            try
            {
                if (node != null)
                {
                    _children.Remove(node);
                    Nodes.Remove(node);
                    node.Remove();
                    UpdateNode();
                }
                
            }
            catch (Exception ex) { MessageBox.Show("RemoveChild: " + ex); }
        }

        
        /// <summary>
        /// Updates the tree node if a child has been added.
        /// </summary>
        public void UpdateNode()
        {
            //return;
            ShowChildren(false);
            ShowChildren(true);
        }


        /// <summary>
        /// Sets all underlying children/nodes to either checked or unchecked based on the input parameter check
        /// </summary>
        /// <param name="check">Whether the childs should be checked or not</param>
        public void CheckChildren(bool check)
        {
            foreach(TreeNode child in Nodes)
            {
                // Sets the child to the check status, the underlyding nodes of "child" are updated by the AfterCheck-event
                child.Checked = check;
                
            }

        }

        /// <summary>
        /// Shows or hides the children of this node in the list
        /// </summary>
        /// <param name="add">If the children should be added or removed from the list</param>
        public void ShowChildren(bool add)
        {
            // Add the nodes to the tree
            if (add)
            {
                foreach (NodeModel tn in _children)
                {
                    Checked = true;
                    if (!Nodes.Contains(tn))
                    {
                        Nodes.Add(tn);
                        tn.Checked = true;
                        tn.ShowChildren(true);
                    }

                    tn.Expand();
                }
            }
            // Remove the nodes from the tree
            else
            {
                Nodes.Clear();
            }
        }

        public string Description
        {
            get { return _description; }
            set { _description = value; }
        }

        public new string Name
        {
            get
            {
                return Text;
            }
            set 
            { 
                Text = value;
            }
        }

        public override string ToString()
        {
            return Text;
        }

        public string Type
        {
            get { return _type; }
            set { _type = value; }
        }

        public string ChildType
        {
            get { return _childtype; }
            set { _childtype = value; }
        }
    }
    
    [Serializable]
    public class InputItem : NodeModel 
    {
        public InputItem(string name, string description)
            : base(name, description)
        {
            ImageIndex = 0;
            SelectedImageIndex = 0;
            Type = "Input item";
            ChildType = "Failure mode";
            
        }

        public InputItem()  
            : this("", "")  
        { }

        public InputItem(SerializationInfo info, StreamingContext ctxt) : base(info, ctxt)
        {
            ImageIndex = 0;
            SelectedImageIndex = 0;
            Type = "Input item";
            ChildType = "Failure mode";
        }

        public override NodeModel AddChild(string name, string description)
        {
            FailureMode fm = new FailureMode(name, description);
            AddChild(fm);
            return fm;
        }

        public override NodeModel AddEmptyChild()
        {
            FailureMode fm = new FailureMode("New failure mode", "Enter description");
            AddChild(fm);
            return fm;
        }

        public void AddChild(FailureMode fm)
        {
            _children.Add(fm);
            fm.Expand();

        }
    }

    [Serializable]
    public class FailureMode : NodeModel
    {
        public FailureMode(string name, string description)
            : base(name, description)
        {
            ImageIndex = 1;
            SelectedImageIndex = 1;
            Type = "Failure mode";
            ChildType = "Barrier";
        }

        public FailureMode()
            : this("", "")
        { }

        public FailureMode(SerializationInfo info, StreamingContext ctxt) : base(info, ctxt)
        {
            ImageIndex = 1;
            SelectedImageIndex = 1;
            Type = "Failure mode";
            ChildType = "Barrier";
        }

        public override NodeModel AddChild(string name, string description)
        {
            Barrier b = new Barrier(name, description);
            AddChild(b);
            return b;
        }

        public override NodeModel AddEmptyChild()
        {
            Barrier b = new Barrier("New Barrier", "Enter description");
            AddChild(b);
            return b;
        }

        public void AddChild(Barrier b)
        {
            _children.Add(b);
            b.Expand();
        }
    }

    [Serializable]
    public class Barrier : NodeModel
    {
        public Barrier(string name, string description)
            : base(name, description)
        {
            ImageIndex = 2;
            SelectedImageIndex = 2;
            Type = "Barrier";
            ChildType = "Action";
        }

        public Barrier()
            : this("", "")    
        { }

        public Barrier(SerializationInfo info, StreamingContext ctxt) : base(info, ctxt)
        {
            ImageIndex = 2;
            SelectedImageIndex = 2;
            Type = "Barrier";
            ChildType = "Action";
        }

        public override NodeModel AddChild(string name, string description)
        {
            Action a = new Action(name, description);
            AddChild(a);
            return a;
        }

        public override NodeModel AddEmptyChild()
        {
            Action a = new Action("New Action", "Enter description");
            AddChild(a);
            return a;
        }

        public void AddChild(Action a)
        {
            _children.Add(a);
            a.Expand();
        }
    }

    [Serializable]
    public class Action : NodeModel
    {
        public Action(string name, string description)
            : base(name, description)
        {
            ImageIndex = 3;
            SelectedImageIndex = 3;
            Type = "Action";
            ChildType = "Test";
        }

        public Action()
            : this("", "")    
        { }

        public Action(SerializationInfo info, StreamingContext ctxt) : base(info, ctxt)
        {
            ImageIndex = 3;
            SelectedImageIndex = 3;
            Type = "Action";
            ChildType = "Test";
        }

        public override NodeModel AddChild(string name, string description)
        {
            Test t = new Test(name, description);
            AddChild(t);
            return t;
        }

        public override NodeModel AddEmptyChild()
        {
            Test t = new Test("New Test", "Enter description");
            AddChild(t);
            return t;
        }

        public void AddChild(Test t)
        {
            _children.Add(t);
            t.Expand();
        }
    }

    [Serializable]
    public class Test : NodeModel
    {
        public Test(string name, string description)
            : base(name, description)
        {
            ImageIndex = 4;
            SelectedImageIndex = 4;
            Type = "Test";
            ChildType = "";
        }

        public Test()
            : this("", "")    
        { }

        public Test(SerializationInfo info, StreamingContext ctxt) : base(info, ctxt)
        {
            ImageIndex = 4;
            SelectedImageIndex = 4;
            Type = "Test";
            ChildType = "";
        }

        public override NodeModel AddChild(string name, string description)
        {
            // Nothing here, no nodes under Test
            return null;
        }

        public override NodeModel AddEmptyChild()
        {
            // Nothing here, no nodes under Test
            return null;
        }
    }
   
}
